@@ -62,7 +62,7 @@ class ConfigDialog : public QDialog, private Ui::ConfigDialog
     void browseSsdir();
     void browseSfdir();
     void browseIncdir();
-    void browseDefaultCsd();
+//    void browseDefaultCsd();
     void browseFavorite();
     void browsePythonDir();
     void browseLogFile();
@@ -79,6 +79,9 @@ class ConfigDialog : public QDialog, private Ui::ConfigDialog
     void selectTextColor();
     void selectBgColor();
 
+    void clearTemplate();
+    void defaultTemplate();
+
   signals:
 //    void changeFont();
 
