@@ -39,9 +39,12 @@ CsoundEngine::CsoundEngine()
   ud->PERF_STATUS = 0;
   ud->cs = this;
   ud->threaded = true;
-  ud->csound = NULL;
+  ud->csound = 0;
   ud->perfThread = 0;
   ud->mouseValues.resize(6); // For _MouseX _MouseY _MouseRelX _MouseRelY _MouseBut1 and _MouseBut2 channels
+#ifdef QCS_PYTHONQT
+  ud->m_pythonCallback = "";
+#endif
 
   pFields = (MYFLT *) calloc(QCS_EVENTS_MAX_PFIELDS, sizeof(MYFLT)); // Maximum number of p-fields for events
 
@@ -156,7 +159,7 @@ void CsoundEngine::inputValueCallback(CSOUND *csound,
   CsoundUserData *ud = (CsoundUserData *) csoundGetHostData(csound);
 //  if (ud->cs->isRunning()) {
     QString name = QString(channelName);
-    ud->cs->perfMutex.lock();
+    ud->cs->perfMutex.lock(); //FIXME get rid of this mutex
     if (name.startsWith('$')) { // channel is a string channel
       char *string = (char *) value;
       // FIMXE: check string length
@@ -191,10 +194,11 @@ void CsoundEngine::inputValueCallback(CSOUND *csound,
 //  }
 }
 
-void CsoundEngine::makeGraphCallback(CSOUND *csound, WINDAT *windat, const char *name)
+void CsoundEngine::makeGraphCallback(CSOUND *csound, WINDAT *windat, const char * /*name*/)
 {
   CsoundUserData *ud = (CsoundUserData *) csoundGetHostData(csound);
   // Csound reuses windat, so it is not guaranteed to be unique
+  // name seems not to be used.
 //  qDebug() << "CsoundEngine::makeGraph() " << windat << "  " << name<< windat->windid;
   ud->wl->appendCurve(windat);
 }
@@ -284,94 +288,92 @@ void CsoundEngine::csThread(void *data)
 //                       &udata->values,
 //                       &udata->stringValues);
   if (!udata->useInvalue) {
-//    CsoundChannelListEntry **channelList;
-//    int numChannels = csoundListChannels(udata->csound, channelList);
-//    csoundDeleteChannelList(udata->csound, *channelList);
-//    for (int j = 0; j < numChannels; j++) {
-//      qDebug() << (char*) channelList[j] << "--";
-//    }
-//    writeWidgetValues(udata);
-//    readWidgetValues(udata);
+//        csoundDeleteChannelList(udata->csound, *channelList);
+    writeWidgetValues(udata);
+    readWidgetValues(udata);
   }
-  udata->cs->processEventQueue();  // FIXME: This function locks a mutex, not ideal here
+  udata->cs->processEventQueue();
   (udata->ksmpscount)++;
+#ifdef QCS_PYTHONQT
+  if (!udata->m_pythonCallback.isEmpty()) {
+    if (udata->m_pythonCallbackCounter >= udata->m_pythonCallbackSkip) {
+      udata->m_pythonConsole->evaluate(udata->m_pythonCallback, false);
+      udata->m_pythonCallbackCounter = 0;
+    }
+    else {
+      udata->m_pythonCallbackCounter++;
+    }
+  }
+#endif
 }
 
 void CsoundEngine::readWidgetValues(CsoundUserData *ud)
 {
   MYFLT* pvalue;
-//   CsoundChannelListEntry **lst;
-//   CsoundChannelListEntry *chan;
-//   int num = csoundListChannels(ud->csound, lst);
-//   for (int i = 0; i < num; i++) {
-//     chan = lst[i];
-//     if (chan) {  //Not sure why this check is needed here....
-//       if (chan->type & (CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL)) {
-//         if(csoundGetChannelPtr(ud->csound, &pvalue, chan->name, 0) == 0) {
-//           *pvalue = (MYFLT) ud->qcs->values[i];
-//         }
-//       }
-//       else if (chan->type & (CSOUND_INPUT_CHANNEL | CSOUND_STRING_CHANNEL)) {
-//         if(csoundGetChannelPtr(ud->csound, &pvalue, chan->name, 0) == 0) {
-//           char *string = (char *) pvalue;
-//           strcpy(string, ud->qcs->stringValues[i].toStdString().c_str());
-//         }
-//       }
-//     }
-//   }
-//   csoundDeleteChannelList(ud->csound, *lst);
 
-  for (int i = 0; i < ud->channelNames.size(); i++) {
-    if(csoundGetChannelPtr(ud->csound, &pvalue, ud->channelNames[i].toStdString().c_str(),
+  for (int i = 0; i < ud->inputChannelNames.size(); i++) {
+    if(csoundGetChannelPtr(ud->csound, &pvalue, ud->inputChannelNames[i].toLocal8Bit().constData(),
         CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
-      *pvalue = (MYFLT) ud->values[i];
+      bool modified = false;
+      double value = ud->wl->getValueForChannel(ud->inputChannelNames[i]);
+      if (modified) {
+        *pvalue = (MYFLT) value;
+      }
+//      *pvalue = (MYFLT) (ud->inputValues[i].toDouble());
     }
-    if(csoundGetChannelPtr(ud->csound, &pvalue, ud->channelNames[i].toStdString().c_str(),
+    else if(csoundGetChannelPtr(ud->csound, &pvalue, ud->inputChannelNames[i].toLocal8Bit().constData(),
        CSOUND_INPUT_CHANNEL | CSOUND_STRING_CHANNEL) == 0) {
+      bool modified = false;
       char *string = (char *) pvalue;
-      strcpy(string, ud->stringValues[i].toStdString().c_str());
+      QString value = ud->wl->getStringForChannel(ud->inputChannelNames[i]);
+      if (modified) {
+        strcpy(string, value.toLocal8Bit().constData());
+      }
+    }
+    else {
+      qDebug() << "CsoundEngine::readWidgetValues invalid input channel: " << ud->inputChannelNames[i];
     }
   }
   //FIXME check if mouse tracking is active
-  if(csoundGetChannelPtr(ud->csound, &pvalue, "_MouseX",
-        CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
-      *pvalue = (MYFLT) ud->mouseValues[0];
-  }
-  else if(csoundGetChannelPtr(ud->csound, &pvalue, "_MouseY",
-        CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
-      *pvalue = (MYFLT) ud->mouseValues[1];
-  }
-  else if(csoundGetChannelPtr(ud->csound, &pvalue, "_MouseRelX",
-        CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
-      *pvalue = (MYFLT) ud->mouseValues[2];
-  }
-  else if(csoundGetChannelPtr(ud->csound, &pvalue, "_MouseRelY",
-        CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
-      *pvalue = (MYFLT) ud->mouseValues[3];
-  }
-  else if(csoundGetChannelPtr(ud->csound, &pvalue, "_MouseBut1",
-        CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
-      *pvalue = (MYFLT) ud->mouseValues[4];
-  }
-  else if(csoundGetChannelPtr(ud->csound, &pvalue, "_MouseBut2",
-        CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
-      *pvalue = (MYFLT) ud->mouseValues[5];
-  }
+//  if(csoundGetChannelPtr(ud->csound, &pvalue, "_MouseX",
+//        CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
+//      *pvalue = (MYFLT) ud->mouseValues[0];
+//  }
+//  else if(csoundGetChannelPtr(ud->csound, &pvalue, "_MouseY",
+//        CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
+//      *pvalue = (MYFLT) ud->mouseValues[1];
+//  }
+//  else if(csoundGetChannelPtr(ud->csound, &pvalue, "_MouseRelX",
+//        CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
+//      *pvalue = (MYFLT) ud->mouseValues[2];
+//  }
+//  else if(csoundGetChannelPtr(ud->csound, &pvalue, "_MouseRelY",
+//        CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
+//      *pvalue = (MYFLT) ud->mouseValues[3];
+//  }
+//  else if(csoundGetChannelPtr(ud->csound, &pvalue, "_MouseBut1",
+//        CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
+//      *pvalue = (MYFLT) ud->mouseValues[4];
+//  }
+//  else if(csoundGetChannelPtr(ud->csound, &pvalue, "_MouseBut2",
+//        CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
+//      *pvalue = (MYFLT) ud->mouseValues[5];
+//  }
 }
 
 void CsoundEngine::writeWidgetValues(CsoundUserData *ud)
 {
 //   qDebug("qutecsound::writeWidgetValues");
    MYFLT* pvalue;
-   for (int i = 0; i < ud->channelNames.size(); i++) {
-     if (ud->channelNames[i] != "") {
-       if(csoundGetChannelPtr(ud->csound, &pvalue, ud->channelNames[i].toStdString().c_str(),
+   for (int i = 0; i < ud->outputChannelNames.size(); i++) {
+     if (ud->outputChannelNames[i] != "") {
+       if(csoundGetChannelPtr(ud->csound, &pvalue, ud->outputChannelNames[i].toLocal8Bit().constData(),
           CSOUND_OUTPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0) {
-         ud->wl->setValue(i,*pvalue);
+         ud->wl->setValue(ud->outputChannelNames[i],*pvalue);
        }
-       else if(csoundGetChannelPtr(ud->csound, &pvalue, ud->channelNames[i].toStdString().c_str(),
+       else if(csoundGetChannelPtr(ud->csound, &pvalue, ud->outputChannelNames[i].toLocal8Bit().constData(),
          CSOUND_OUTPUT_CHANNEL | CSOUND_STRING_CHANNEL) == 0) {
-         ud->wl->setValue(i,QString((char *)pvalue));
+         ud->wl->setValue(ud->outputChannelNames[i],QString((char *)pvalue));
        }
      }
    }
@@ -393,6 +395,17 @@ void CsoundEngine::setWidgetLayout(WidgetLayout *wl)
 {
   ud->wl = wl;
 //  connect(wl, SIGNAL(destroyed()), this, SLOT(widgetLayoutDestroyed()));
+  // Key presses on widget layout and console are passed to the engine
+  connect(wl, SIGNAL(keyPressed(QString)),
+          this, SLOT(keyPressForCsound(QString)));
+  connect(wl, SIGNAL(keyReleased(QString)),
+          this, SLOT(keyReleaseForCsound(QString)));
+
+  // Register scopes and graphs to pass them the engine's user data
+  connect(wl, SIGNAL(registerScope(QuteScope*)),
+          this,SLOT(registerScope(QuteScope*)));
+  connect(wl, SIGNAL(registerGraph(QuteGraph*)),
+          this,SLOT(registerGraph(QuteGraph*)));
   dispatchQueues(); // starts queue dispatcher timer
 }
 
@@ -567,7 +580,6 @@ void CsoundEngine::stop()
 {
   stopRecording();
   stopCsound();
-  emit stopSignal();
 }
 
 void CsoundEngine::pause()
@@ -594,9 +606,9 @@ int CsoundEngine::startRecording(int sampleformat, QString fileName)
     break;
   }
   qDebug("start recording: %s", fileName.toStdString().c_str());
-  outfile = new SndfileHandle(fileName.toStdString().c_str(), SFM_WRITE, format, channels, sampleRate);
+  m_outfile = new SndfileHandle(fileName.toStdString().c_str(), SFM_WRITE, format, channels, sampleRate);
   // clip instead of wrap when converting floats to ints
-  outfile->command(SFC_SET_CLIPPING, NULL, SF_TRUE);
+  m_outfile->command(SFC_SET_CLIPPING, NULL, SF_TRUE);
   samplesWritten = 0;
   m_recording = true;
 
@@ -647,17 +659,47 @@ int CsoundEngine::runCsound()
   ud->csound=csoundCreate(0);
 #endif
 
-  // Message Callbacks must be set before compile, otherwise some information is missed
-  if (ud->threaded) {
-    csoundSetMessageCallback(ud->csound, &CsoundEngine::messageCallbackThread);
+  // Environment variables must be set before csoundCompile
+  if (m_options.sadirActive){
+    int ret = csoundSetGlobalEnv("SADIR", m_options.sadir.toLocal8Bit().constData());
+    if (ret != 0) {
+      qDebug() << "CsoundEngine::runCsound() Error setting SADIR";
+    }
   }
   else {
-    csoundSetMessageCallback(ud->csound, &CsoundEngine::messageCallbackNoThread);
+    csoundSetGlobalEnv("SADIR", "");
+  }
+  if (m_options.ssdirActive){
+    int ret = csoundSetGlobalEnv("SSDIR", m_options.ssdir.toLocal8Bit().constData());
+    if (ret != 0) {
+      qDebug() << "CsoundEngine::runCsound() Error setting SSDIR";
+    }
+  }
+  else {
+    csoundSetGlobalEnv("SSDIR", "");
+  }
+  if (m_options.sfdirActive){
+    int ret = csoundSetGlobalEnv("SFDIR", m_options.sfdir.toLocal8Bit().constData());
+    if (ret != 0) {
+      qDebug() << "CsoundEngine::runCsound() Error setting SFDIR";
+    }
+  }
+  else {
+    csoundSetGlobalEnv("SFDIR", "");
+  }
+  if (m_options.incdirActive){
+    int ret = csoundSetGlobalEnv("INCDIR", m_options.incdir.toLocal8Bit().constData());
+    if (ret != 0) {
+      qDebug() << "CsoundEngine::runCsound() Error setting INCDIR";
+    }
+  }
+  else {
+    csoundSetGlobalEnv("INCDIR", "");
   }
   if (m_options.opcodedirActive) {
     // csoundGetEnv must be called after Compile or Precompile,
     // But I need to set OPCODEDIR before compile.... So I can't know keep the old OPCODEDIR
-    csoundSetGlobalEnv("OPCODEDIR", m_options.opcodedir.toLocal8Bit());
+    csoundSetGlobalEnv("OPCODEDIR", m_options.opcodedir.toLocal8Bit().constData());
   }
 #ifdef Q_OS_MAC
   else {
@@ -670,11 +712,20 @@ int CsoundEngine::runCsound()
 #endif
     // TODO is this check robust enough? what if the standard library is not used? is it likely it is not?
     if (QFile::exists(stdopcode)) {
-      csoundSetGlobalEnv("OPCODEDIR", opcodedir.toLocal8Bit());
+      csoundSetGlobalEnv("OPCODEDIR", opcodedir.toLocal8Bit().constData());
     }
   }
 #endif
-  csoundReset(ud->csound);
+
+  // Message Callbacks must be set before compile, otherwise some information is missed
+  if (ud->threaded) {
+    csoundSetMessageCallback(ud->csound, &CsoundEngine::messageCallbackThread);
+  }
+  else {
+    csoundSetMessageCallback(ud->csound, &CsoundEngine::messageCallbackNoThread);
+  }
+
+//  csoundReset(ud->csound);
   csoundSetHostData(ud->csound, (void *) ud);
   csoundPreCompile(ud->csound);  //Need to run PreCompile to create the FLTK_Flags global variable
 
@@ -687,6 +738,7 @@ int CsoundEngine::runCsound()
   }
   else {
     //       qDebug("play() FLTK Disabled");
+    csoundSetGlobalEnv("CS_OMIT_LIBS", "fluidOpcodes,virtual,widgets");
     *((int*) csoundQueryGlobalVariable(ud->csound, "FLTK_Flags")) = 3;
   }
 
@@ -698,22 +750,6 @@ int CsoundEngine::runCsound()
     csoundSetExitGraphCallback(ud->csound, &CsoundEngine::exitGraphCallback);
   }
 
-  if (ud->enableWidgets) {
-    if (ud->useInvalue) {
-      csoundSetInputValueCallback(ud->csound, &CsoundEngine::inputValueCallback);
-      csoundSetOutputValueCallback(ud->csound, &CsoundEngine::outputValueCallback);
-    }
-    else {
-      // Not really sure that this is worth the trouble, as it
-      // is used only with chnsend and chnrecv which are unfinished
-      //         qDebug() << "csoundSetChannelIOCallback";
-      //         csoundSetChannelIOCallback(csound, &qutecsound::ioCallback);
-    }
-  }
-  else {
-    csoundSetInputValueCallback(ud->csound, NULL);
-    csoundSetOutputValueCallback(ud->csound, NULL);
-  }
   csoundSetCallback(ud->csound,
                     &CsoundEngine::keyEventCallback,
                     (void *) ud, CSOUND_CALLBACK_KBD_EVENT);
@@ -733,14 +769,57 @@ int CsoundEngine::runCsound()
     emit (errorLines(getErrorLines()));
     return -3;
   }
+  for (int i = 0; i < argc; i++) {
+    qDebug() << argv[i];
+  }
   ud->zerodBFS = csoundGet0dBFS(ud->csound);
   ud->sampleRate = csoundGetSr(ud->csound);
   ud->numChnls = csoundGetNchnls(ud->csound);
   ud->outputBufferSize = csoundGetKsmps(ud->csound);
   ud->ksmpscount = 0;
 
+  if (ud->enableWidgets) {
+    ud->inputChannelNames.clear();
+    ud->inputValues.clear();
+    ud->outputChannelNames.clear();
+    ud->outputValues.clear();
+    ud->outputValueChanged.clear();
+    if (ud->useInvalue){
+      csoundSetInputValueCallback(ud->csound, &CsoundEngine::inputValueCallback);
+      csoundSetOutputValueCallback(ud->csound, &CsoundEngine::outputValueCallback);
+    }
+    else {
+      MYFLT *pvalue;
+      ud->channelList = (CsoundChannelListEntry **) malloc(sizeof(CsoundChannelListEntry *));
+      ud->numChannels = csoundListChannels(ud->csound, ud->channelList);
+      CsoundChannelListEntry *chnls = *ud->channelList;
+      for (int i = 0; i < ud->numChannels; i++) {
+        if (csoundGetChannelPtr(ud->csound, &pvalue, chnls->name,
+                                0) & CSOUND_INPUT_CHANNEL) {
+          ud->inputChannelNames << QString(chnls->name);
+          ud->inputValues << QVariant();
+        }
+        else if (csoundGetChannelPtr(ud->csound, &pvalue, chnls->name,
+                                     0) & CSOUND_OUTPUT_CHANNEL) {
+          ud->outputChannelNames << QString(chnls->name);
+          ud->outputValues << QVariant();
+          ud->outputValueChanged << false;
+        }
+        chnls++;
+      }
+      // Not really sure that this is worth the trouble, as it
+      // is used only with chnsend and chnrecv which are unfinished:
+      //         qDebug() << "csoundSetChannelIOCallback";
+      //         csoundSetChannelIOCallback(csound, &qutecsound::ioCallback);
+    }
+  }
+  else {
+    csoundSetInputValueCallback(ud->csound, NULL);
+    csoundSetOutputValueCallback(ud->csound, NULL);
+  }
+
   if (ud->threaded) {
-    ud->perfThread = new CsoundPerformanceThread(ud->csound);
+    ud->perfThread = new QCsPerfThread(ud->csound);
     ud->perfThread->SetProcessCallback(CsoundEngine::csThread, (void*)ud);
 //    qDebug() << "qutecsound::runCsound perfThread->Play";
     ud->perfThread->Play();
@@ -754,11 +833,11 @@ int CsoundEngine::runCsound()
     }
     ud->PERF_STATUS = 0;
     csoundStop(ud->csound);
-    csoundSetMessageCallback(ud->csound, 0); // Does this fix the messages that appear when closing QCS?
-    csoundCleanup(ud->csound);
+    csoundSetMessageCallback(ud->csound, 0); // This seems to fix the messages that appear when closing QCS?
 #ifdef QCS_DESTROY_CSOUND
     csoundDestroy(ud->csound);
 #else
+    csoundCleanup(ud->csound);
 #endif
     flushQueues();  // To flush pending queues
 #ifdef MACOSX_PRE_SNOW
@@ -783,13 +862,15 @@ void CsoundEngine::stopCsound()
   if (ud->threaded) {
 //    perfThread->ScoreEvent(0, 'e', 0, 0);
     if (ud->perfThread != 0) {
-      ud->perfThread->Stop();
-//      qDebug() << "CsoundEngine::stopCsound() stopped";
-      ud->perfThread->Join();
-      qDebug() << "CsoundEngine::stopCsound() joined";
-      delete ud->perfThread;
+      QCsPerfThread *pt = ud->perfThread;
       ud->perfThread = 0;
+      pt->Stop();
+//      qDebug() << "CsoundEngine::stopCsound() stopped";
+      pt->Join();
+      qDebug() << "CsoundEngine::stopCsound() joined";
+      delete pt;
       flushQueues();
+      emit stopSignal();
     }
   } /*if (ud->threaded)*/
   else {  // in same thread
@@ -799,6 +880,7 @@ void CsoundEngine::stopCsound()
         usleep(100000);
         qApp->processEvents();
       }
+      emit stopSignal();
     }
   }
 #ifdef MACOSX_PRE_SNOW
@@ -807,6 +889,8 @@ void CsoundEngine::stopCsound()
 #endif
 #ifdef QCS_DESTROY_CSOUND
   csoundDestroy(ud->csound);
+#else
+  csoundReset(ud->csound);
 #endif
 }
 
@@ -817,7 +901,7 @@ void CsoundEngine::dispatchQueues()
     QTimer::singleShot(refreshTime, this, SLOT(dispatchQueues()));
     return;
   }
-  if (closing == 1) {
+  if (closing == 1 || closing == -1) {
     closing = -1;
     engineMutex.unlock();
     return;
@@ -844,12 +928,12 @@ void CsoundEngine::dispatchQueues()
   if (!messageQueue.isEmpty() && m_consoleBufferSize > 0 && counter >= m_consoleBufferSize) {
     messageQueue.clear();
     messageQueue << "\nQUTECSOUND: Message buffer overflow. Messages discarded!\n";
-    qDebug() << "CsoundEngine::dispatchQueues() " << m_consoleBufferSize << counter;
+//    qDebug() << "CsoundEngine::dispatchQueues() " << m_consoleBufferSize << counter;
   }
   messageMutex.unlock();
 
   if (ud->threaded && ud->perfThread) {
-    if (ud->perfThread->GetStatus() > 0) {
+    if (ud->perfThread->GetStatus() != 0) {
 //      qDebug() << "CsoundEngine::dispatchQueues() perf finished";
       stop();
     }
@@ -861,7 +945,7 @@ void CsoundEngine::dispatchQueues()
 void CsoundEngine::queueMessage(QString message)
 {
   if (closing != 1) {
-    messageMutex.lock(); // FIXME this is still occasionally causing threading issues as Csound uses it unexpectedly. Confirmed (with dssi?)
+    messageMutex.lock(); // FIXME this is still occasionally causing threading issues as Csound uses it unexpectedly when closing. Confirmed
     messageQueue << message;
     messageMutex.unlock();
   }
@@ -896,7 +980,7 @@ void CsoundEngine::recordBuffer()
 {
   if (m_recording) {
     if (ud->audioOutputBuffer.copyAvailableBuffer(recBuffer, bufferSize)) {
-      int samps = outfile->write(recBuffer, bufferSize);
+      int samps = m_outfile->write(recBuffer, bufferSize);
       samplesWritten += samps;
     }
     else {
@@ -905,7 +989,7 @@ void CsoundEngine::recordBuffer()
     recordTimer.singleShot(20, this, SLOT(recordBuffer()));
   }
   else { //Stop recording
-    delete outfile;
+    delete m_outfile;
     qDebug("Recording stopped. Written %li samples", samplesWritten);
   }
 }
@@ -928,3 +1012,20 @@ bool CsoundEngine::isRecording()
   return m_recording;
 }
 
+CSOUND *CsoundEngine::getCsound()
+{
+  return ud->csound;
+}
+
+#ifdef QCS_PYTHONQT
+void CsoundEngine::registerProcessCallback(QString func, int skipPeriods)
+{
+  ud->m_pythonCallback = func;
+  ud->m_pythonCallbackSkip = skipPeriods;
+}
+
+void CsoundEngine::setPythonConsole(PythonConsole *pc)
+{
+  ud->m_pythonConsole = pc;
+}
+#endif
