@@ -21,32 +21,17 @@
 */
 
 #include "documentview.h"
-#include "highlighter.h"
 #include "findreplace.h"
 #include "opentryparser.h"
 #include "node.h"
 #include "types.h"
+
+#include "highlighter.h"
 #include "texteditor.h"
 
 DocumentView::DocumentView(QWidget * parent, OpEntryParser *opcodeTree) :
-    QScrollArea(parent),  m_opcodeTree(opcodeTree)
+    BaseView(parent,opcodeTree)
 {
-  mainEditor = new TextEditor(this);
-  scoreEditor = new TextEditor(this);
-  optionsEditor = new TextEditor(this);
-  filebEditor = new TextEditor(this);
-  versionEditor = new TextEditor(this);
-  licenceEditor = new TextEditor(this);
-  otherEditor = new TextEditor(this);
-  widgetEditor = new TextEditor(this);
-  ladspaEditor = new TextEditor(this);
-  editors << mainEditor << scoreEditor << optionsEditor << filebEditor
-      << versionEditor << licenceEditor << otherEditor << widgetEditor
-      << ladspaEditor;
-  splitter = new QSplitter(this);
-  splitter->setOrientation(Qt::Vertical);
-  splitter->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
-  splitter->setContextMenuPolicy (Qt::NoContextMenu);
   for (int i = 0; i < editors.size(); i++) {
     connect(editors[i], SIGNAL(textChanged()), this, SLOT(setModified()));
     splitter->addWidget(editors[i]);
@@ -54,12 +39,7 @@ DocumentView::DocumentView(QWidget * parent, OpEntryParser *opcodeTree) :
     connect(editors[i], SIGNAL(customContextMenuRequested(QPoint)),
             this, SLOT(createContextMenu(QPoint)));
   }
-  QStackedLayout *l = new QStackedLayout(this);
-  l->addWidget(splitter);
-  setLayout(l);
   setFocusProxy(mainEditor);  // for comment action from main application
-
-  m_mode = 0;
   internalChange = false;
 
 //  m_highlighter = new Highlighter();
@@ -75,18 +55,15 @@ DocumentView::DocumentView(QWidget * parent, OpEntryParser *opcodeTree) :
 //  connect(scoreEditor, SIGNAL(cursorPositionChanged()),
 //          this, SLOT(syntaxCheck()));
 
-  setViewMode(0);
-
   errorMarked = false;
   m_isModified = false;
-  m_highlighter.setDocument(mainEditor->document());
 
   syntaxMenu = new MySyntaxMenu(mainEditor);
 //  syntaxMenu->setFocusPolicy(Qt::NoFocus);
   syntaxMenu->setAutoFillBackground(true);
   QPalette p =syntaxMenu-> palette();
   p.setColor(QPalette::WindowText, Qt::blue);
-  p.setColor(QPalette::Active, static_cast<QPalette::ColorRole>(9), Qt::yellow);
+  p.setColor(static_cast<QPalette::ColorRole>(9), Qt::yellow);
   syntaxMenu->setPalette(p);
   connect(syntaxMenu,SIGNAL(keyPressed(QString)),
           mainEditor, SLOT(insertPlainText(QString)));
@@ -102,90 +79,6 @@ DocumentView::~DocumentView()
 //  delete m_highlighter;
 }
 
-void DocumentView::setViewMode(int mode)
-{
-//  if (m_viewMode == mode)
-//    return;
-  m_viewMode = mode;
-  hideAllEditors();
-
-  // TODO implement modes properly
-  switch (m_viewMode) {
-    case 0: // csd without extra sections
-      mainEditor->show();
-      break;
-    case 1: // full plain text
-      mainEditor->show();
-      break;
-    default:
-      mainEditor->setVisible(m_viewMode & 2);
-      scoreEditor->setVisible(m_viewMode & 4);
-      optionsEditor->setVisible(m_viewMode & 8);
-      filebEditor->setVisible(m_viewMode & 16);
-      versionEditor->setVisible(m_viewMode & 32);
-      licenceEditor->setVisible(m_viewMode & 64);
-      otherEditor->setVisible(m_viewMode & 128);
-      widgetEditor->setVisible(m_viewMode & 256);
-  }
-
-}
-
-void DocumentView::setFileType(int mode)
-{
-  m_highlighter.setMode(mode);
-  m_mode = mode;
-}
-
-void DocumentView::setFont(QFont font)
-{
-  for (int i = 0; i < editors.size(); i++) {
-    editors[i]->setFont(font);
-  }
-}
-
-void DocumentView::setFontPointSize(float size)
-{
-  for (int i = 0; i < editors.size(); i++) {
-    editors[i]->setFontPointSize(size);
-  }
-}
-
-void DocumentView::setTabWidth(int width)
-{
-  for (int i = 0; i < editors.size(); i++) {
-    editors[i]->setTabStopWidth(width);
-  }
-}
-
-void DocumentView::setTabStopWidth(int width)
-{
-  for (int i = 0; i < editors.size(); i++) {
-    editors[i]->setTabStopWidth(width);
-  }
-}
-
-void DocumentView::setLineWrapMode(QTextEdit::LineWrapMode mode)
-{
-  for (int i = 0; i < editors.size(); i++) {
-    editors[i]->setLineWrapMode(mode);
-  }
-}
-
-void DocumentView::setAutoComplete(bool autoComplete)
-{
-  m_autoComplete = autoComplete;
-}
-
-void DocumentView::setColorVariables(bool color)
-{
-  m_highlighter.setColorVariables(color);
-}
-
-void DocumentView::setOpcodeNameList(QStringList list)
-{
-  m_highlighter.setOpcodeNameList(list);
-}
-
 bool DocumentView::isModified()
 {
   return m_isModified;
@@ -203,15 +96,60 @@ void DocumentView::setModified(bool mod)
   m_isModified = mod;
 }
 
-void DocumentView::setOpcodeTree(OpEntryParser *opcodeTree)
+void DocumentView::insertText(QString text, int section)
 {
-  m_opcodeTree = opcodeTree;
+  if (section == -1 || section < 0) {
+    section = 0;  // TODO implment for multiple views
+  }
+  if (section < editors.size()) {
+    QTextCursor cursor = editors[section]->textCursor();
+    cursor.insertText(text);
+    editors[section]->setTextCursor(cursor);
+  }
 }
 
-void DocumentView::setFullText(QString text)
+void DocumentView::setBasicText(QString text)
 {
-  editors[0]->setText(text);
-  setModified(false);
+  QTextCursor cursor = editors[0]->textCursor();
+  cursor.select(QTextCursor::Document);
+  cursor.insertText(text);
+  editors[0]->setTextCursor(cursor);  // TODO implment for multiple views
+}
+
+void DocumentView::setOrc(QString text)
+{
+  QTextCursor cursor = editors[0]->textCursor();
+  QString csdText = getBasicText();
+  if (csdText.contains("<CsInstruments>") and csdText.contains("</CsInstruments>")) {
+    QString preText = csdText.mid(0, csdText.indexOf("<CsInstruments>") + 15);
+    QString postText = csdText.mid(csdText.lastIndexOf("</CsInstruments>"));
+    if (!text.startsWith("\n")) {
+      text.prepend("\n");
+    }
+    if (!text.endsWith("\n")) {
+      text.append("\n");
+    }
+    csdText = preText + text + postText;
+    setBasicText(csdText);
+  }
+}
+
+void DocumentView::setSco(QString text)
+{
+  QTextCursor cursor = editors[0]->textCursor();
+  QString csdText = getBasicText();
+  if (csdText.contains("<CsScore>") and csdText.contains("</CsScore>")) {
+    QString preText = csdText.mid(0, csdText.indexOf("<CsScore>") + 9);
+    QString postText = csdText.mid(csdText.lastIndexOf("</CsScore>"));
+    if (!text.startsWith("\n")) {
+      text.prepend("\n");
+    }
+    if (!text.endsWith("\n")) {
+      text.append("\n");
+    }
+    csdText = preText + text + postText;
+    setBasicText(csdText);
+  }
 }
 
 void DocumentView::setLadspaText(QString text)
@@ -246,6 +184,23 @@ void DocumentView::setLadspaText(QString text)
   edit->moveCursor(QTextCursor::Start);
 }
 
+void DocumentView::setAutoComplete(bool autoComplete)
+{
+  m_autoComplete = autoComplete;
+}
+
+QString DocumentView::getSelectedText(int section)
+{
+  QString text;
+  if (section == -1 || section < 0) {
+    section = 0;  // TODO implment for multiple views
+  }
+  if (section < editors.size()) {
+    text = editors[section]->textCursor().selectedText();
+  }
+  return text;
+}
+
 QString DocumentView::getFullText()
 {
   QString text;
@@ -269,15 +224,33 @@ QString DocumentView::getBasicText()
   return text;
 }
 
-QString DocumentView::getOrcText()
-{// Without tags
-  qDebug() << "DocumentView::getOrcText() not implemented and will crash!";
+QString DocumentView::getOrc()
+{
+  QString text = "";
+  if (m_viewMode < 2) { // A single editor (orc and sco are not split)
+    text = mainEditor->toPlainText();
+    text = text.mid(text.lastIndexOf("<CsInstruments>" )+ 15);
+    qDebug() << text;
+    text.remove(text.lastIndexOf("</CsInstruments>"), text.size());
+  }
+  else {
+    text = mainEditor->toPlainText();
+  }
+  return text;
 }
 
-QString DocumentView::getScoText()
+QString DocumentView::getSco()
 {
-  // Without tags
-  qDebug() << "DocumentView::getScoText() not implemented and will crash!";
+  QString text = "";
+  if (m_viewMode < 2) { // A single editor (orc and sco are not split)
+    text = mainEditor->toPlainText();
+    text = text.mid(text.lastIndexOf("<CsScore>") + 9);
+    text.remove(text.lastIndexOf("</CsScore>"), text.size());
+  }
+  else {
+    text = scoreEditor->toPlainText();
+  }
+  return text;
 }
 
 QString DocumentView::getOptionsText()
@@ -339,14 +312,57 @@ int DocumentView::currentLine()
 QString DocumentView::wordUnderCursor()
 {
   QTextCursor cursor = editors[0]->textCursor();
-  cursor.select(QTextCursor::WordUnderCursor);
   QString word = cursor.selectedText();
-  if (word.startsWith('#')) {
-    word.remove(0,1);
+  if (word.isEmpty()) {
+    cursor.select(QTextCursor::WordUnderCursor);
+    word = cursor.selectedText();
   }
   return word;
 }
 
+QString DocumentView::getActiveSection()
+{
+  QTextCursor cursor = editors[0]->textCursor();
+  cursor.select(QTextCursor::LineUnderCursor);
+  bool sectionStart = cursor.selectedText().simplified().startsWith("##");
+  while (!sectionStart && !cursor.anchor() == 0) {
+    cursor.movePosition(QTextCursor::PreviousBlock);
+    cursor.select(QTextCursor::LineUnderCursor);
+    sectionStart = cursor.selectedText().simplified().startsWith("##");
+  }
+  int start = cursor.anchor();
+  cursor = editors[0]->textCursor();
+  cursor.movePosition(QTextCursor::NextBlock);
+  cursor.select(QTextCursor::LineUnderCursor);
+  bool sectionEnd = cursor.selectedText().simplified().startsWith("##");
+  while (!sectionEnd && !cursor.atEnd()) {
+    cursor.movePosition(QTextCursor::NextBlock);
+    cursor.select(QTextCursor::LineUnderCursor);
+    sectionEnd = cursor.selectedText().simplified().startsWith("##");
+  }
+  cursor.movePosition(QTextCursor::EndOfLine);
+  cursor.setPosition(start, QTextCursor::KeepAnchor);
+  editors[0]->setTextCursor(cursor);
+  QString section = cursor.selectedText();
+  section.replace(QChar(0x2029), QChar('\n'));
+  return section;
+}
+
+QString DocumentView::getActiveText()
+{
+  QTextCursor cursor = editors[0]->textCursor();
+  QString selection = cursor.selectedText();
+  if (selection == "") {
+    cursor.movePosition(QTextCursor::StartOfLine, QTextCursor::MoveAnchor);
+    cursor.movePosition(QTextCursor::EndOfLine, QTextCursor::KeepAnchor);
+    selection = cursor.selectedText();
+
+  }
+  selection.replace(QChar(0x2029), QChar('\n'));
+  return selection;
+}
+
+
 //void DocumentView::updateDocumentModel()
 //{
 //  // this should update the document model when needed
@@ -362,24 +378,22 @@ QString DocumentView::wordUnderCursor()
 
 void DocumentView::syntaxCheck()
 {
-  // TODO implement for multiple views
+  // TODO implment for multiple views
 
   int line = currentLine();
   emit(lineNumberSignal(line));
 
-  if (m_mode == 0 || m_mode == 3) {  // CSD or ORC mode
-    QTextCursor cursor = mainEditor->textCursor();
-    cursor.select(QTextCursor::LineUnderCursor);
-    QStringList words = cursor.selectedText().split(QRegExp("\\b"));
-    foreach(QString word, words) {
-      // We need to remove all not possibly opcode
-      word.remove(QRegExp("[^\\d\\w]"));
-      if (!word.isEmpty()) {
-        QString syntax = m_opcodeTree->getSyntax(word);
-        if(!syntax.isEmpty()) {
-          emit(opcodeSyntaxSignal(syntax));
-          return;
-        }
+  QTextCursor cursor = mainEditor->textCursor();
+  cursor.select(QTextCursor::LineUnderCursor);
+  QStringList words = cursor.selectedText().split(QRegExp("\\b"));
+  foreach(QString word, words) {
+    // We need to remove all not possibly opcode
+    word.remove(QRegExp("[^\\d\\w]"));
+    if (!word.isEmpty()) {
+      QString syntax = m_opcodeTree->getSyntax(word);
+      if(!syntax.isEmpty()) {
+        emit(opcodeSyntaxSignal(syntax));
+        return;
       }
     }
   }
@@ -599,10 +613,17 @@ void DocumentView::findString(QString query)
   }
 }
 
+void DocumentView::evaluate()
+{
+  emit evaluate(getActiveText());
+}
+
 void DocumentView::createContextMenu(QPoint pos)
 {
   QMenu *menu = editors[0]->createStandardContextMenu();
   menu->addSeparator();
+  menu->addAction(tr("Evaluate Selection"), this, SLOT(evaluate()));
+  menu->addSeparator();
   QMenu *opcodeMenu = menu->addMenu("Opcodes");
   QMenu *mainMenu = 0;
   QMenu *subMenu;
@@ -711,10 +732,20 @@ void DocumentView::comment()
   if (!cursor.atBlockStart()) {
     cursor.movePosition(QTextCursor::StartOfLine, QTextCursor::KeepAnchor);
   }
+  int start = cursor.selectionStart();
   QString text = cursor.selectedText();
+  if (text.startsWith(commentChar)) {
+    uncomment();
+    return;
+  }
   text.prepend(commentChar);
   text.replace(QChar(QChar::ParagraphSeparator), QString("\n" + commentChar));
+  if (text.endsWith("\n" + commentChar) ) {
+    text.chop(1);
+  }
   cursor.insertText(text);
+  cursor.setPosition(start);
+  cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor, text.size());
   editors[0]->setTextCursor(cursor);
 }
 
@@ -740,11 +771,15 @@ void DocumentView::uncomment()
     cursor.movePosition(QTextCursor::StartOfLine, QTextCursor::KeepAnchor);
     text = cursor.selectedText();
   }
-  if (text.startsWith(commentChar))
+  if (text.startsWith(commentChar)) {
     text.remove(0,1);
+  }
+  int start = cursor.selectionStart();
   text.replace(QChar(QChar::ParagraphSeparator), QString("\n"));
   text.replace(QString("\n" + commentChar), QString("\n")); //TODO make more robust
   cursor.insertText(text);
+  cursor.setPosition(start);
+  cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor, text.size());
   editors[0]->setTextCursor(cursor);
 }
 
@@ -769,11 +804,16 @@ void DocumentView::indent()
   if (!cursor.atBlockStart()) {
     cursor.movePosition(QTextCursor::StartOfLine, QTextCursor::KeepAnchor);
   }
+  int start = cursor.selectionStart();
   QString text = cursor.selectedText();
   text.prepend(indentChar);
   text.replace(QChar(QChar::ParagraphSeparator), "\n" + indentChar);
-  text.replace(QChar(QChar::ParagraphSeparator), "\n" + indentChar);
+  if (text.endsWith("\n" + indentChar) ) {
+    text.chop(1);
+  }
   cursor.insertText(text);
+  cursor.setPosition(start);
+  cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor, text.size());
   editors[0]->setTextCursor(cursor);
 }
 
@@ -797,6 +837,7 @@ void DocumentView::unindent()
   if (!cursor.atBlockStart()) {
     cursor.movePosition(QTextCursor::StartOfLine, QTextCursor::KeepAnchor);
   }
+  int start = cursor.selectionStart();
   QString text = cursor.selectedText();
   while (indentChar == "    "  && text.startsWith("\t")) {
     text.remove(0,1);
@@ -807,6 +848,8 @@ void DocumentView::unindent()
   text.replace(QChar(QChar::ParagraphSeparator), QString("\n"));
   text.replace("\n" + indentChar, QString("\n")); //TODO make more robust
   cursor.insertText(text);
+  cursor.setPosition(start);
+  cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor, text.size());
   editors[0]->setTextCursor(cursor);
 }
 
@@ -870,14 +913,19 @@ void DocumentView::markErrorLines(QList<QPair<int, QString> > lines)
       cur.mergeCharFormat(errorFormat);
       internalChange = true;
       editors[0]->setTextCursor(cur);
+      cur.movePosition(QTextCursor::PreviousBlock, QTextCursor::MoveAnchor);
+      cur.movePosition(QTextCursor::PreviousBlock, QTextCursor::MoveAnchor);
+      cur.movePosition(QTextCursor::PreviousBlock, QTextCursor::MoveAnchor);
+      cur.movePosition(QTextCursor::PreviousBlock, QTextCursor::MoveAnchor);
+      cur.movePosition(QTextCursor::PreviousBlock, QTextCursor::MoveAnchor); // go up 5 lines
       cur.movePosition(QTextCursor::StartOfBlock, QTextCursor::MoveAnchor);
     }
     else {
       qDebug() << "DocumentView::markErrorLines: Error line text doesn't match\n" << text;
     }
   }
-  internalChange = true;
-  editors[0]->setTextCursor(cur);
+//  internalChange = true;
+//  editors[0]->setTextCursor(cur);
   errorMarked = true;
   if (!originallyMod) {
     editors[0]->document()->setModified(false);
@@ -992,14 +1040,6 @@ QString DocumentView::changeToInvalue(QString text)
 //  syntaxMenu->show();
 //}
 
-void DocumentView::hideAllEditors()
-{
-  for (int i = 0; i < editors.size(); i++) {
-    editors[i]->hide();
-    splitter->handle(i)->hide();
-  }
-}
-
 void DocumentView::destroySyntaxMenu()
 {
   syntaxMenu->hide();
