@@ -266,20 +266,41 @@ QString EventSheet::getLine(int number, bool scaleTempo, bool storeNumber, bool
   return line;
 }
 
+QList< QList<QVariant> > EventSheet::getData()
+{
+  // TODO this can be made a lot more efficient
+  QList< QList<QVariant> > data;
+  for (int i = 0; i < this->rowCount(); i++) {
+    QList<QVariant> row;
+    for (int j = 0; j < this->columnCount(); j++) {
+      QTableWidgetItem * item = this->item(i, j);
+      if (item != 0) { // Item is not empty
+        row.append(item->data(Qt::DisplayRole));
+      }
+      else {
+        row.append(QVariant());
+      }
+    }
+    data << row;
+  }
+  return data;
+}
+
 void EventSheet::setFromText(QString text, int rowOffset, int columnOffset, int numRows, int numColumns, bool noHistoryMark)
 {
   // Separataion is stored in UserRole of items
   // remember to treat comments and formulas properly
   QStringList lines = text.split("\n");
-  int nRows = numRows == 0 ? lines.size() : numRows;  // 0 pastes only available data
-  nRows = (numRows == -1  && nRows <  this->rowCount()) ?  this->rowCount() : nRows; //-1 pastes all of numColumns even if not in data
-  if (this->rowCount() <nRows + rowOffset) {
+  int nRows = 0; // Number of actual rows to process
+  // numRows = 0 : don't remove rows, only add if necessary. numRows = -1 limit rows to the ones in text
+  nRows = numRows <= 0 ? lines.size() : numRows;
+  if (this->rowCount() < nRows + rowOffset || numRows == -1) {
     this->setRowCount(nRows + rowOffset);
   }
   for (int i = 0; i < nRows; i++) {
-    if (nRows != 0 && i >= nRows) {  // Only paste up to a certain number of rows if not 0
-      break;
-    }
+//    if (nRows != 0 && i >= nRows) {  // Only paste up to a certain number of rows if not 0
+//      break;
+//    }
     QString line = "";
     if (i < lines.size()) {
       line = lines[i].trimmed(); //Remove whitespace from start and end
@@ -318,6 +339,16 @@ void EventSheet::setFromText(QString text, int rowOffset, int columnOffset, int
     this->setRowCount(1);
 }
 
+void EventSheet::setCell(int row, int column, QVariant value)
+{
+  QTableWidgetItem * item = this->item(row, column);
+  if (item == 0) {
+    item = new QTableWidgetItem();
+    this->setItem(row, column, item);
+  }
+  item->setData(Qt::DisplayRole, value);
+}
+
 void EventSheet::setDebug(bool debug)
 {
   m_debug = debug;
@@ -538,7 +569,7 @@ void EventSheet::copy(bool cut)
 
 void EventSheet::paste()
 {
-  qDebug() << "EventSheet::paste() text = " << qApp->clipboard()->text();
+//  qDebug() << "EventSheet::paste() text = " << qApp->clipboard()->text();
   QModelIndexList list = this->selectedIndexes();
   QList<int> selectedRows;
   QList<int> selectedColumns;
@@ -862,7 +893,7 @@ void EventSheet::runScript(QString name)
   moduleStream << file.readAll();
   file.close();
   module.close();
-  qDebug() << module.fileName();
+  qDebug() << "EventSheet::runScript module " << module.fileName();
 
   QFile script(tempDir.absolutePath() + QDir::separator() + name.mid(name.lastIndexOf("/") + 1));
   script.open(QFile::WriteOnly | QIODevice::Text);
