@@ -30,31 +30,32 @@ class QuteComboBox : public QuteWidget
   Q_OBJECT
   public:
     QuteComboBox(QWidget *parent);
-
     ~QuteComboBox();
 
-    virtual void loadFromXml(QString xmlText);
-    virtual void setValue(double value); // Current item select index
-    virtual double getValue();
     virtual QString getWidgetLine();
     virtual QString getCabbageLine();
     virtual QString getWidgetXmlText();
     virtual QString getWidgetType();
-    void setSize(int size);
     void setText(QString text);  //Text for this widget is the item list separated by commas
-    void popUpMenu(QPoint pos);
-    QString itemList();
+    void clearItems();
+    void addItem(QString text, double value, QString stringvalue);
+    virtual void refreshWidget();
+    virtual void applyInternalProperties();
 
   protected:
-    virtual void contextMenuEvent(QContextMenuEvent* event);
     virtual void applyProperties();
     virtual void createPropertiesDialog();
 
   private:
-    int m_size;
+    QString itemList();
+    QStringList stringValues;
+//    int m_size;
     QLineEdit *text;
     QLineEdit *line;
 
+  private slots:
+    void indexChanged(int value);
+
 };
 
 #endif
