@@ -34,15 +34,15 @@ class QuteConsole : public QuteWidget
 
     ~QuteConsole();
 
-    virtual void loadFromXml(QString xmlText);
     virtual QString getWidgetLine();
     virtual QString getWidgetXmlText();
     virtual QString getWidgetType();
     virtual void setWidgetGeometry(int x,int y,int width,int height);
-//     void popUpMenu(QPoint pos);
 
     void appendMessage(QString message);
     void scrollToEnd();
+
+  protected:
 };
 
 #endif
