@@ -32,10 +32,12 @@ class QuteDummy : public QuteWidget
     QuteDummy(QWidget *parent);
     ~QuteDummy();
 
-    virtual void loadFromXml(QString xmlText);
+    virtual QString getWidgetLine();
     virtual QString getWidgetXmlText();
     virtual QString getWidgetType();
 
+  protected:
+
 };
 
 #endif
