@@ -49,26 +49,27 @@ class QuteScope : public QuteWidget
 
     ~QuteScope();
 
-    virtual void loadFromXml(QString xmlText);
     virtual QString getWidgetLine();
     virtual QString getWidgetXmlText();
     virtual QString getWidgetType();
-    virtual void setWidgetGeometry(int x,int y,int width,int height);
+//    virtual void setWidgetGeometry(int x,int y,int width,int height);
     virtual void createPropertiesDialog();
     void setType(QString type);
     void setValue(double value); //Value for this widget indicates zoom level
-    void setChannel(int channel);
+//    void setChannel(int channel);
     void setUd(CsoundUserData *ud);
-    void updateLabel();
+
+    virtual void applyInternalProperties();
 
   protected:
+//    QString m_type;
+//    int m_channel;
+//    int m_zoom;
     QLabel * m_label;
-    QString m_type;
-    int m_channel;
-    int m_zoom;
     QComboBox *typeComboBox;
     QComboBox *channelBox;
-    QSpinBox *decimationBox;
+    QDoubleSpinBox *zoomxBox;
+    QDoubleSpinBox *zoomyBox;
     ScopeParams *m_params;
     DataDisplay *m_dataDisplay;
     ScopeData *m_scopeData;
@@ -77,6 +78,10 @@ class QuteScope : public QuteWidget
 
     virtual void resizeEvent(QResizeEvent * event);
     virtual void applyProperties();
+    QReadWriteLock scopeLock;
+
+  private:
+    void updateLabel();
 
   public slots:
     void updateData();
@@ -122,7 +127,7 @@ class ScopeParams
 {
   public:
     ScopeParams(CsoundUserData *ud, QGraphicsScene *scene, ScopeWidget *widget,
-                QMutex *mutex, int width, int height)
+                QReadWriteLock *mutex, int width, int height)
     {
       this->ud = ud;
       this->scene = scene;
@@ -143,7 +148,7 @@ class ScopeParams
     CsoundUserData *ud;
     QGraphicsScene *scene;
     ScopeWidget *widget;
-    QMutex *mutex;
+    QReadWriteLock *mutex;
     int width;
     int height;
 };
@@ -185,7 +190,7 @@ class DataDisplay
       m_params = params;
     }
     virtual void resize() = 0;
-    virtual void updateData(int channel, int zoom, bool freeze) = 0;
+    virtual void updateData(int channel, double zoomx, double zoomy, bool freeze) = 0;
     virtual void show() = 0;
     virtual void hide() = 0;
 
@@ -203,7 +208,7 @@ class ScopeData : public DataDisplay
     ScopeData(ScopeParams *params);
     virtual ~ScopeData() {}
     virtual void resize();
-    virtual void updateData(int channel, int zoom, bool freeze);
+    virtual void updateData(int channel, double zoomx, double zoomy, bool freeze);
     virtual void show();
     virtual void hide();
 
@@ -222,7 +227,7 @@ class LissajouData : public DataDisplay
     LissajouData(ScopeParams *params);
     virtual ~LissajouData() {}
     virtual void resize();
-    virtual void updateData(int channel, int zoom, bool freeze);
+    virtual void updateData(int channel, double zoomx, double zoomy, bool freeze);
     virtual void show();
     virtual void hide();
 
@@ -241,7 +246,7 @@ class PoincareData : public DataDisplay
     PoincareData(ScopeParams *params);
     virtual ~PoincareData() {}
     virtual void resize();
-    virtual void updateData(int channel, int zoom, bool freeze);
+    virtual void updateData(int channel, double zoomx, double zoomy, bool freeze);
     virtual void show();
     virtual void hide();
 
