@@ -478,7 +478,7 @@ void QuteText::applyProperties()
 #ifdef  USE_WIDGET_MUTEX
   widgetLock.unlock();
 #endif
-  QuteWidget::applyProperties();  //Must be last to make sure the widgetsChanged signal is last
+  QuteWidget::applyProperties();  //Must be last to make sure the widgetChanged signal is last
 }
 
 void QuteText::selectTextColor()
@@ -730,7 +730,7 @@ void QuteLineEdit::createPropertiesDialog()
 //    default:
 //      setProperty("QCS_alignment", "");
 //  }
-//  QuteWidget::applyProperties();  //Must be last to make sure the widgetsChanged signal is last
+//  QuteWidget::applyProperties();  //Must be last to make sure the widgetChanged signal is last
 //}
 
 void QuteLineEdit::textEdited(QString text)
@@ -999,7 +999,7 @@ void QuteScrollNumber::applyProperties()
   widgetLock.unlock();
 #endif
 //  setValue(m_value);
-  QuteText::applyProperties();  //Must be last to make sure the widgetsChanged signal is last
+  QuteText::applyProperties();  //Must be last to make sure the widgetChanged signal is last
 }
 
 void QuteScrollNumber::applyInternalProperties()
@@ -1108,6 +1108,21 @@ void QuteScrollNumber::setValue(double value)
 //  emit widgetChanged(this);
 }
 
+void QuteScrollNumber::setMidiValue(int value)
+{
+  double max = property("QCS_maximum").toDouble();
+  double min = property("QCS_minimum").toDouble();
+  if (max != 99999999999999.0 && min != -999999999999.0) {
+    double newval = min + ((value / 127.0)* (max - min));
+    setValue(newval);
+    QPair<QString, double> channelValue(m_channel, newval);
+    emit newValue(channelValue);
+  }
+  else {
+    qDebug() << "QuteScrollNumber::setMidiValue ranges not set.";
+  }
+}
+
 void QuteScrollNumber::setValueFromWidget(double value)
 {
 //  qDebug() << "QuteScrollNumber::setValueFromWidget";
